@@ -1,6 +1,5 @@
 /*
- * Copyright (C) 2006, 2007, 2008, 2009 Free Software Foundation
- * Copyright (C) 2002 Nikos Mavroyanopoulos
+ * Copyright (C) 2002-2026 Free Software Foundation, Inc.
  *
  * This file is part of LIBTASN1.
  *
@@ -15,12 +14,14 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA
+ * License along with this library; if not, see
+ * <https://www.gnu.org/licenses/>.
  */
 
-#include <int.h>
+#include <config.h>
+
+#include "int.h"
+#include "gstr.h"
 
 /* These function are like strcat, strcpy. They only
  * do bounds checking (they shouldn't cause buffer overruns),
@@ -40,7 +41,7 @@ _asn1_str_cat (char *dest, size_t dest_tot_size, const char *src)
     }
   else
     {
-      if (dest_tot_size - dest_size > 0)
+      if (dest_tot_size > dest_size)
 	{
 	  strncat (dest, src, (dest_tot_size - dest_size) - 1);
 	  dest[dest_tot_size - 1] = 0;
@@ -48,7 +49,8 @@ _asn1_str_cat (char *dest, size_t dest_tot_size, const char *src)
     }
 }
 
-void
+/* Returns the bytes copied (not including the null terminator) */
+unsigned int
 _asn1_str_cpy (char *dest, size_t dest_tot_size, const char *src)
 {
   size_t str_size = strlen (src);
@@ -56,13 +58,18 @@ _asn1_str_cpy (char *dest, size_t dest_tot_size, const char *src)
   if (dest_tot_size > str_size)
     {
       strcpy (dest, src);
+      return str_size;
     }
   else
     {
       if (dest_tot_size > 0)
 	{
-	  strncpy (dest, src, (dest_tot_size) - 1);
-	  dest[dest_tot_size - 1] = 0;
+	  str_size = dest_tot_size - 1;
+	  memcpy (dest, src, str_size);
+	  dest[str_size] = 0;
+	  return str_size;
 	}
+      else
+	return 0;
     }
 }
